@@ -8,13 +8,13 @@ from ...config import Config
 
 
 argument_oar_job_id: click.argument = click.argument(
-    'oar-job-id',
+    "oar-job-id",
     type=int,
-    nargs=1
+    nargs=1,
 )
 
 argument_oar_job_id_list: click.argument = click.argument(
-    'oar-job-ids',
+    "oar-job-ids",
     type=int,
-    nargs=-1
+    nargs=-1,
 )
