@@ -19,7 +19,7 @@ def init() -> None:
     """
     print_info(
         text="Creating a brand new remi project.",
-        bold=True
+        bold=True,
     )
     create_default_project()
 
@@ -36,27 +36,31 @@ def setup(hostname: str) -> None:
     config: Config = Config()
 
     print_info(
-        text=f'Setting up remote for project: {config.project_name}',
-        bold=True
+        text=f"Setting up remote for project: {config.project_name}",
+        bold=True,
     )
 
     paths_to_create: str = config.project_remote_path
-    paths_to_create += '/{' + ','.join(
-        [
-            config.output_path,
-            config.notebooks_path,
-            config.oarsub_log_path
-        ]
-    ) + '}'
+    paths_to_create += (
+        "/{"
+        + ",".join(
+            [
+                config.output_path,
+                config.notebooks_path,
+                config.oarsub_log_path,
+            ]
+        )
+        + "}"
+    )
 
     # Create the project directory on the remote and the output folder within it.
     create_remote_directory(
         path=paths_to_create,
-        hostname=hostname
+        hostname=hostname,
     )
     push_inria()
 
-    print_info('Setup complete !')
+    print_info("Setup complete !")
 
 
 @click.command()
@@ -67,12 +71,12 @@ def update() -> None:
     if prompt_user_yes_no("Do you wish to update 'remi' to the latest version ?"):
         run_local_cmd(
             command=[
-                'pip',
-                'install',
-                '-U',
-                'remote-inria'
+                "pip",
+                "install",
+                "-U",
+                "remote-inria",
             ],
-            check=True
+            check=True,
         )
     else:
         print_info("All right the, aborting.")
